package heapStacking;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class HeapStacker {
	private boolean initialized = false;
	private int heapDepth;
	private int heapLength;
	private int totalAmount;
	private double[] inputQualities;
	private int qualityCount;
	private int amountFactor;
	private int[][] stackedHeights;
	private double[] reclaimQualities;
	private int[] reclaimAmounts;
	private int[] sliceHeights;
	private int maxHeight;
	private int maxLayers;
	
	private void Error(String message) {
		System.out.println("Error: " + message);
	}
	
	public HeapStacker(String qualityInputFileName, int heapLength, int heapDepth, int amountFactor, int maxLayers) {
		initialized = false;
		this.heapLength = heapLength;
		this.heapDepth = heapDepth;
		this.amountFactor = amountFactor;
		this.maxLayers = maxLayers;
		
		stackedHeights = new int[heapLength][heapDepth];
		reclaimAmounts = new int[heapLength];
		reclaimQualities = new double[heapLength];
		sliceHeights = new int[heapLength];

		clearHeap();
		
		ArrayList<Double> input = new ArrayList<Double>();
		totalAmount = 0;
		
		File file = new File(qualityInputFileName);
			
		if (!file.exists()) {
			Error("Quality input file does not exist (\"" + qualityInputFileName + "\").");
			return;
		}
		
		try {
			FileInputStream fstream = new FileInputStream(file);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));

			String s;
			
			while ((s = br.readLine()) != null) {
				input.add(Double.parseDouble(s));
			}

			br.close();
			in.close();
			fstream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		qualityCount = input.size();
		totalAmount = qualityCount * amountFactor;
		inputQualities = new double[qualityCount];
		
		for (int i = 0; i < qualityCount; i++) {
			inputQualities[i] = input.get(i);
		}
		
		initialized = true;
	}
	
	public void clearHeap() {
		maxHeight = 0;
		
		for (int i = 0; i < heapLength; i++) {
			for (int j = 0; j < heapDepth; j++) {
				stackedHeights[i][j] = 0;
			}

			reclaimAmounts[i] = 0;
			reclaimQualities[i] = 0.0f;
			sliceHeights[i] = 0;
		}
	}

	public double[] evaluatePositions(double[] positions) {
		if (!initialized) {
			Error("Stacker not initialized");
			return null;
		}
		
		clearHeap();
		
		int depthCenter = heapDepth / 2;
		int tempAmount = 0;
		double pixelOffset = depthCenter;
		double pixelWidth = heapLength - depthCenter * 2;

		for (int i = 0; i < qualityCount; i++) {
			double q = inputQualities[i];
			
			for (int j = 0; j < amountFactor; j++) {
				// Negative qualities are interpreted as empty belt
				if (q >= 0.0) {
					double partPosition = (double)(positions.length - 1) * (double)tempAmount / (double)totalAmount;
					int positionIndex = (int)partPosition;
					double v = partPosition - (double)positionIndex;
					double newPosition = (positions[positionIndex] * (1.0 - v) + positions[positionIndex + 1] * v) * pixelWidth + pixelOffset;
					
					int i1 = (int)(newPosition + 0.5);
					int j1 = depthCenter;

					int minHeightI;
					int minHeightJ;
					int minHeight = stackedHeights[i1][j1];
					
					do {
						minHeightI = -1;
						minHeightJ = -1;
						
						int ti = i1;
						int tj = j1;
				
						// Check all 8 fall possibilities 
						for (int k = 0; k < 8; k++) {
							switch (k) {
								case 0:
									// Left
									ti = i1;
									tj = j1 - 1;
									break;
					
								case 1:
									// Top
									ti = i1 - 1;
									tj = j1;
									break;
					
								case 2:
									// Right
									ti = i1;
									tj = j1 + 1;
									break;
					
								case 3:
									// Bottom
									ti = i1 + 1;
									tj = j1;
									break;
					
								case 4:
									// Top Left
									ti = i1 - 1;
									tj = j1 - 1;
									break;
					
								case 5:
									// Top Right
									ti = i1 - 1;
									tj = j1 + 1;
									break;
					
								case 6:
									// Bottom Right
									ti = i1 + 1;
									tj = j1 + 1;
									break;
									
								case 7:
									// Bottom Left
									ti = i1 + 1;
									tj = j1 - 1;
									break;
									
							}
							
							if (ti >= 0 && ti < heapLength && tj >= 0 && tj < heapDepth) {
								int lh = stackedHeights[ti][tj];
							
								if (lh < minHeight) {
									minHeightI = ti;
									minHeightJ = tj;
									minHeight = lh;
								}
							}
						}

						if (minHeightI >= 0) {
							i1 = minHeightI;
							j1 = minHeightJ;
						}
					} while (minHeightI >= 0);
					
					minHeight++;
					
					stackedHeights[i1][j1] = minHeight;
					
					int qIndex = i1 - minHeight;
					
					if (qIndex < 0) {
						qIndex = 0;
					}
					
					if (qIndex >= heapLength) {
						qIndex = heapLength - 1;
					}
					
					reclaimQualities[qIndex] = reclaimQualities[qIndex] + q;
					reclaimAmounts[qIndex]++;
					
					if (minHeight > maxHeight) {
						maxHeight = minHeight;
					}
					
					if (minHeight > sliceHeights[i1]) {
						sliceHeights[i1] = minHeight;
					}
				}
				
				tempAmount = tempAmount + 1;
			}
		}
		
		double meanQuality = 0;
		int heightMin = 999999999;
		int heightMax = 0;
		int localMaxHeight;
		int avgMaxHeight = 0;
		int avgMaxHeightCount = 0;
		
		for (int i = 0; i < heapLength; i++) {
			localMaxHeight = sliceHeights[i];
			
			meanQuality = meanQuality + reclaimQualities[i];
			
			if (i >= heapDepth / 2 && i <= heapLength - heapDepth / 2) {
				if (localMaxHeight < heightMin) {
					heightMin = localMaxHeight;
				}
				
				if (localMaxHeight > heightMax) {
					heightMax = localMaxHeight;
				}
				
				avgMaxHeight = avgMaxHeight + localMaxHeight;
				avgMaxHeightCount++;
			}
		}

		// Slices count with amount > 0 
		int nDash = 0;
		
		for (int i = 0; i < heapLength; i++) {
			if (reclaimAmounts[i] > 0) {
				reclaimQualities[i] = reclaimQualities[i] / (double)reclaimAmounts[i];
				nDash++;
			} else {
				reclaimQualities[i] = 0.0f;
			}
		}
		
		meanQuality = meanQuality / (double)totalAmount;
		
		double upper = 0;
		
		for (int i = 0; i < heapLength; i++) {
			upper = upper + (double)reclaimAmounts[i] * (double)(reclaimQualities[i] - meanQuality) * (double)(reclaimQualities[i] - meanQuality);
		}
		
		double[] fitness = {upper / ((double)(nDash - 1) * (double)totalAmount / nDash),
							(double)(heightMax - heightMin) / ((double)avgMaxHeight / (double)avgMaxHeightCount)};
		
		return fitness;
	}
	
	public double[] evaluateSpeeds(double[] speeds) {
		if (!initialized) {
			Error("Stacker not initialized");
			return null;
		}
		
		clearHeap();
		
		int depthCenter = heapDepth / 2;
		int tempAmount = 0;
		double pixelOffset = depthCenter;
		double pixelWidth = heapLength - depthCenter * 2;
		double currentPixelPosition = pixelOffset;
		double maxSpeedPerParticle = (double)maxLayers * pixelWidth / (double)totalAmount;
		double amountPerSpeed = (double)totalAmount / (double)speeds.length;
		int speedIndex = 0;

		for (int i = 0; i < qualityCount; i++) {
			double q = inputQualities[i];
			
			for (int j = 0; j < amountFactor; j++) {
				speedIndex = (int)Math.floor((double)tempAmount / amountPerSpeed);
				
				// Negative qualities are interpreted as empty belt
				if (q >= 0.0) {
					double normalizedPosition = Math.abs(currentPixelPosition - pixelOffset) / pixelWidth;
					int directionState = (int)normalizedPosition;
					double subPosition = normalizedPosition - (double)directionState;
					double newPosition = ((directionState % 2 == 1) ? 1.0 - subPosition : subPosition) * pixelWidth + pixelOffset;
					
					int i1 = (int)Math.round(newPosition);
					int j1 = depthCenter;

					int minHeightI;
					int minHeightJ;
					int minHeight = stackedHeights[i1][j1];
					
					do {
						minHeightI = -1;
						minHeightJ = -1;
						
						int ti = i1;
						int tj = j1;
				
						// Check all 8 fall possibilities 
						for (int k = 0; k < 8; k++) {
							switch (k) {
								case 0:
									// Left
									ti = i1;
									tj = j1 - 1;
									break;
					
								case 1:
									// Top
									ti = i1 - 1;
									tj = j1;
									break;
					
								case 2:
									// Right
									ti = i1;
									tj = j1 + 1;
									break;
					
								case 3:
									// Bottom
									ti = i1 + 1;
									tj = j1;
									break;
					
								case 4:
									// Top Left
									ti = i1 - 1;
									tj = j1 - 1;
									break;
					
								case 5:
									// Top Right
									ti = i1 - 1;
									tj = j1 + 1;
									break;
					
								case 6:
									// Bottom Right
									ti = i1 + 1;
									tj = j1 + 1;
									break;
									
								case 7:
									// Bottom Left
									ti = i1 + 1;
									tj = j1 - 1;
									break;
									
							}
							
							if (ti >= 0 && ti < heapLength && tj >= 0 && tj < heapDepth) {
								int lh = stackedHeights[ti][tj];
							
								if (lh < minHeight) {
									minHeightI = ti;
									minHeightJ = tj;
									minHeight = lh;
								}
							}
						}

						if (minHeightI >= 0) {
							i1 = minHeightI;
							j1 = minHeightJ;
						}
					} while (minHeightI >= 0);
					
					minHeight++;
					
					stackedHeights[i1][j1] = minHeight;
					
					int qIndex = i1 - minHeight;
					
					if (qIndex < 0) {
						qIndex = 0;
					}
					
					if (qIndex >= heapLength) {
						qIndex = heapLength - 1;
					}
					
					reclaimQualities[qIndex] = reclaimQualities[qIndex] + q;
					reclaimAmounts[qIndex]++;
					
					if (minHeight > maxHeight) {
						maxHeight = minHeight;
					}
					
					if (minHeight > sliceHeights[i1]) {
						sliceHeights[i1] = minHeight;
					}
				}

				currentPixelPosition = currentPixelPosition + speeds[speedIndex] * maxSpeedPerParticle;
				tempAmount = tempAmount + 1;
			}
		}
		
		double meanQuality = 0;
		int heightMin = 999999999;
		int heightMax = 0;
		int localMaxHeight;
		int avgMaxHeight = 0;
		int avgMaxHeightCount = 0;
		
		for (int i = 0; i < heapLength; i++) {
			localMaxHeight = sliceHeights[i];
			
			meanQuality = meanQuality + reclaimQualities[i];
			
			if (i >= heapDepth / 2 && i <= heapLength - heapDepth / 2) {
				if (localMaxHeight < heightMin) {
					heightMin = localMaxHeight;
				}
				
				if (localMaxHeight > heightMax) {
					heightMax = localMaxHeight;
				}
				
				avgMaxHeight = avgMaxHeight + localMaxHeight;
				avgMaxHeightCount++;
			}
		}

		// Slices count with amount > 0 
		int nDash = 0;
		
		for (int i = 0; i < heapLength; i++) {
			if (reclaimAmounts[i] > 0) {
				reclaimQualities[i] = reclaimQualities[i] / (double)reclaimAmounts[i];
				nDash++;
			} else {
				reclaimQualities[i] = 0.0f;
			}
		}
		
		meanQuality = meanQuality / (double)totalAmount;
		
		double upper = 0;
		
		for (int i = 0; i < heapLength; i++) {
			upper = upper + (double)reclaimAmounts[i] * (double)(reclaimQualities[i] - meanQuality) * (double)(reclaimQualities[i] - meanQuality);
		}
		
		double[] fitness = {upper / ((double)(nDash - 1) * (double)totalAmount / nDash),
							(double)(heightMax - heightMin) / ((double)avgMaxHeight / (double)avgMaxHeightCount)};
		
		return fitness;
	}
}
