package heapStacking.problems;

import java.io.FileNotFoundException;
import java.io.IOException;
import jmetal.core.Solution;
import jmetal.core.Variable;
import jmetal.encodings.variable.ArrayReal;
import jmetal.util.JMException;

@SuppressWarnings("serial")
public class HSP7 extends HSPPositionsBase {
	public HSP7(String problemName, String baseDirectory, String qualityFile) throws FileNotFoundException, ClassNotFoundException, IOException {
		super(problemName, baseDirectory, qualityFile);
		
	    numberOfObjectives_ = 2;
	    numberOfConstraints_= 1;
	}
	
	// Position representation
	// No repairing, only constraint measure vs. advanced repaired version

	@Override
	public void evaluate(Solution solution) throws JMException {
		Variable[] variables = solution.getDecisionVariables();
		double[] strategy = new double[numberOfVariables_];

		for (int i = 0; i < numberOfVariables_; i++) {
			strategy[i] = ((ArrayReal)variables[0]).array_[i];
		}
		
		double[] fitness = stacker.evaluatePositions(strategy);

		solution.setObjective(0, fitness[0]);
		solution.setObjective(1, fitness[1]);

		Log(fitness[0], fitness[1]);
	}
	
	@Override
	public void evaluateConstraints(Solution solution) throws JMException {
		Variable[] variables = solution.getDecisionVariables();
		double[] strategy = new double[numberOfVariables_];
		boolean violated = false;

		boolean changed;
		boolean first = true;
		
		do {
			changed = false;
			
			for (int i = 0; i < numberOfVariables_; i++) {
				if (first) {
					strategy[i] = ((ArrayReal)variables[0]).array_[i];
				}
	
				if (i > 0) {
					if (strategy[i] >= strategy[i - 1]) {
						if (strategy[i] - strategy[i - 1] > maxDistance) {
							double distance = strategy[i] - strategy[i - 1] - maxDistance;
							strategy[i] = strategy[i] - distance / 2.0;
							strategy[i - 1] = strategy[i - 1] + distance / 2.0;
							violated = true;
							changed = true;
						}
					} else {
						if (strategy[i - 1] - strategy[i] > maxDistance) {
							double distance = strategy[i - 1] - strategy[i] - maxDistance;
							strategy[i] = strategy[i] + distance / 2.0;
							strategy[i - 1] = strategy[i - 1] - distance / 2.0;
							violated = true;
							changed = true;
						}
					}
				}
			}
			
			first = false;
		} while(changed);

		double totalDistance = 0.0;
		
		for (int i = 0; i < numberOfVariables_; i++) {
			totalDistance = totalDistance + Math.abs(strategy[i] - ((ArrayReal)variables[0]).array_[i]);
		}
	        
	    solution.setOverallConstraintViolation(totalDistance);    
	    solution.setNumberOfViolatedConstraint(violated ? 1 : 0);
	}
}