package heapStacking.problems;

import heapStacking.HeapStacker;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import jmetal.core.Problem;
import jmetal.encodings.solutionType.ArrayRealSolutionType;

@SuppressWarnings("serial")
public abstract class HSPBase extends Problem {
	protected HeapStacker stacker;
	protected int maxEvaluations;
	protected int evaluated;
	protected DecimalFormat dfPercentage;
	protected DecimalFormat dfTime;
	protected DecimalFormat dfFitness;
	protected List<Long> timeList;
	protected Long lastPrint;
	protected String problemName;
	
	public HSPBase(String problemName, String baseDirectory, String qualityFile) throws ClassNotFoundException, FileNotFoundException, IOException {
		this.problemName = problemName;
		
		Properties properties = new Properties();
		properties.load(new FileInputStream(baseDirectory + "\\config.ini"));
				
		stacker = new HeapStacker(	baseDirectory + qualityFile,
									Integer.parseInt(properties.getProperty("heap_length")),
									Integer.parseInt(properties.getProperty("heap_depth")),
									Integer.parseInt(properties.getProperty("amount_factor")),
									Integer.parseInt(properties.getProperty("max_layers")));
		
		maxEvaluations = Integer.parseInt(properties.getProperty("max_evaluations")) * Integer.parseInt(properties.getProperty("independent_runs"));
		evaluated = 0;
		lastPrint = 0L;

		dfPercentage = new DecimalFormat("000.00");
		dfTime = new DecimalFormat("0.0");
		dfFitness = new DecimalFormat("0.0000");
		timeList = new ArrayList<Long>();
			
	    numberOfVariables_ = Integer.parseInt(properties.getProperty("variable_count"));
	    problemName_ = problemName;

	    upperLimit_ = new double[numberOfVariables_];
	    lowerLimit_ = new double[numberOfVariables_];
	    
	    solutionType_ = new ArrayRealSolutionType(this);
	}

	protected void Log(double f0, double f1) {
		long stop = System.currentTimeMillis();
		
		timeList.add(stop);
		
		while (timeList.size() > 500) {
			timeList.remove(0);
		}

		evaluated++;
		
		if (stop - lastPrint > 3000 || maxEvaluations - evaluated == 0) {
			double msPerCalc = (double)(stop - timeList.get(0)) / (double)timeList.size();
			
			// seconds
			double left = (double)(maxEvaluations - evaluated) * msPerCalc / 1000.0;
			String leftString;
			
			if (left > 180.0) {
				// minutes
				left = left / 60.0;
				
				if (left > 120.0) {
					// hours
					left = left / 60.0;
					leftString = dfTime.format(left) + " hours";
				} else {
					leftString = dfTime.format(left) + " minutes";
				}
			} else {
				leftString = dfTime.format(left) + " seconds";
			}
	
			System.out.println(problemName + ": " + dfPercentage.format(100.0f * (double)evaluated / (double)maxEvaluations) + "% | Evaluated: " + evaluated + " | Latest Values: " + dfFitness.format(f0) + "; " + dfFitness.format(f1) + (evaluated > 1 ? " | Avg. evaluation: " + dfTime.format(msPerCalc) + " ms | Time Left: " + leftString : ""));
			
			lastPrint = stop;
		}
	}
}