package heapStacking.study;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;

import jmetal.metaheuristics.nsgaII.*;
import jmetal.operators.crossover.Crossover;
import jmetal.operators.crossover.CrossoverFactory;
import jmetal.operators.mutation.Mutation;
import jmetal.operators.mutation.MutationFactory;
import jmetal.operators.selection.Selection;
import jmetal.operators.selection.SelectionFactory;
import jmetal.core.*;
import jmetal.experiments.Settings;
import jmetal.util.JMException;

public class HeapStackingSettingsSSNSGAII extends Settings {
	public int populationSize_; 
	public int maxEvaluations_;
	public double mutationProbability_;
	public double crossoverProbability_;
	public double mutationDistributionIndex_;
	public double crossoverDistributionIndex_;
	
	public HeapStackingSettingsSSNSGAII(String baseDirectory, String problem) throws JMException, FileNotFoundException, IOException {
		super(problem);
		
		Object[] problemParams = {problem, baseDirectory, "quality_input\\" + problem.split("_")[1] + ".txt"};
		
		problem_ = (new HeapStackingProblemFactory()).getProblem(problemName_, problemParams);
		
		Properties properties = new Properties();
		properties.load(new FileInputStream(baseDirectory + "\\config.ini"));
				
		populationSize_ = Integer.parseInt(properties.getProperty("population_size")); 
		maxEvaluations_ = Integer.parseInt(properties.getProperty("max_evaluations"));
		mutationProbability_ = Double.parseDouble(properties.getProperty("mutation_probability"));
		mutationDistributionIndex_ = Double.parseDouble(properties.getProperty("mutation_distribution_index"));
		crossoverProbability_ = Double.parseDouble(properties.getProperty("crossover_probability"));
		crossoverDistributionIndex_ = Double.parseDouble(properties.getProperty("crossover_distribution_index"));
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public Algorithm configure() throws JMException {
		Algorithm algorithm;
		Selection selection;
		Crossover crossover;
		Mutation mutation;

		// Operator parameters
		HashMap	parameters;
		
		algorithm = new ssNSGAII(problem_);
		
		// Algorithm parameters
		algorithm.setInputParameter("populationSize", populationSize_);
		algorithm.setInputParameter("maxEvaluations", maxEvaluations_);

		// Mutation and Crossover for Real codification
		parameters = new HashMap();
		parameters.put("probability", crossoverProbability_);
		parameters.put("distributionIndex", crossoverDistributionIndex_);
		crossover = CrossoverFactory.getCrossoverOperator("SBXCrossover", parameters);									

		parameters = new HashMap();
		parameters.put("probability", mutationProbability_);
		parameters.put("distributionIndex", mutationDistributionIndex_);
		mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);												

		// Selection Operator 
		parameters = new HashMap();
		selection = SelectionFactory.getSelectionOperator("BinaryTournament2", parameters);		

		// Add the operators to the algorithm
		algorithm.addOperator("crossover",crossover);
		algorithm.addOperator("mutation",mutation);
		algorithm.addOperator("selection",selection);
		
		return algorithm;
	}
}